"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.FunctionsContext = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _react = _interopRequireWildcard(require("react"));

var _reactApollo = require("react-apollo");

var _store = _interopRequireDefault(require("store/dist/store.modern"));

var _context7 = require("../authentication/context");

var _ApolloClient = require("../apollo/ApolloClient");

var _mutations = require("./mutations");

var _queries = require("./queries");

var FunctionsContext = _react["default"].createContext(null);

exports.FunctionsContext = FunctionsContext;

var FunctionsProvider = function FunctionsProvider(_ref) {
  var children = _ref.children,
      uri = _ref.uri,
      shopifyConfigs = _ref.shopifyConfigs;
  var client = (0, _react.useMemo)(function () {
    return (0, _ApolloClient.createApolloClient)({
      uri: uri
    });
  }, [uri]);

  var authContext = _react["default"].useContext(_context7.AuthContext);

  var _useState = (0, _react.useState)(false),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      isLoading = _useState2[0],
      setLoading = _useState2[1];

  var _useState3 = (0, _react.useState)(null),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      shopifyConfig = _useState4[0],
      setShopifyConfig = _useState4[1];

  var initialShopName = _store["default"].get("next:shopname");

  var shouldSkip = !!initialShopName;

  var _useQuery = (0, _reactApollo.useQuery)(_queries.SHOP_NAME, {
    client: client,
    skip: shouldSkip
  }),
      shopNameData = _useQuery.data;

  if (!shopifyConfig) {
    if (initialShopName) {
      var initialShopifyConfig = shopifyConfigs.find(function (config) {
        return config.shopName === initialShopName;
      });

      if (initialShopifyConfig) {
        setShopifyConfig(initialShopifyConfig);
      }
    }

    if (shopNameData && shopNameData.shopName) {
      _store["default"].set("next:shopname", shopNameData.shopName);

      setShopifyConfig(shopifyConfigs.find(function (config) {
        return config.shopName === shopNameData.shopName;
      }));
    }
  }

  var _useMutation = (0, _reactApollo.useMutation)(_mutations.CUSTOMER_UPDATE, {
    client: client
  }),
      _useMutation2 = (0, _slicedToArray2["default"])(_useMutation, 2),
      customerUpdate = _useMutation2[0],
      _useMutation2$ = _useMutation2[1],
      customerData = _useMutation2$.data,
      isLoadingCustomer = _useMutation2$.loading;

  var _useMutation3 = (0, _reactApollo.useMutation)(_mutations.CUSTOMER_PASSWORD_UPDATE, {
    client: client
  }),
      _useMutation4 = (0, _slicedToArray2["default"])(_useMutation3, 2),
      customerPasswordUpdate = _useMutation4[0],
      _useMutation4$ = _useMutation4[1],
      customerPasswordUpdateData = _useMutation4$.data,
      isLoadingCustomerPassword = _useMutation4$.loading;

  var _useMutation5 = (0, _reactApollo.useMutation)(_mutations.CUSTOMER_ACCOUNT_ACTIVATE, {
    client: client
  }),
      _useMutation6 = (0, _slicedToArray2["default"])(_useMutation5, 2),
      customerAccountActivate = _useMutation6[0],
      _useMutation6$ = _useMutation6[1],
      customerAccountActivateData = _useMutation6$.data,
      isLoadingCustomerActivate = _useMutation6$.loading;

  var _useMutation7 = (0, _reactApollo.useMutation)(_mutations.SUBSCRIBER_CREATE, {
    client: client
  }),
      _useMutation8 = (0, _slicedToArray2["default"])(_useMutation7, 2),
      subscriberCreate = _useMutation8[0],
      _useMutation8$ = _useMutation8[1],
      subdcriberCreateData = _useMutation8$.data,
      isLoadingSubscriberCreate = _useMutation8$.loading;

  var _useMutation9 = (0, _reactApollo.useMutation)(_mutations.SUBSCRIBER_UPDATE, {
    client: client
  }),
      _useMutation10 = (0, _slicedToArray2["default"])(_useMutation9, 2),
      subscriberUpdate = _useMutation10[0],
      _useMutation10$ = _useMutation10[1],
      subdcriberUpdateData = _useMutation10$.data,
      isLoadingSubscriberUpdate = _useMutation10$.loading;

  var _useMutation11 = (0, _reactApollo.useMutation)(_mutations.CUSTOMER_ACCESS_TOKEN_CREATE, {
    client: client
  }),
      _useMutation12 = (0, _slicedToArray2["default"])(_useMutation11, 2),
      customerAccessTokenCreate = _useMutation12[0],
      _useMutation12$ = _useMutation12[1],
      customerAccessToken = _useMutation12$.data,
      isLoadingCustomerAccessToken = _useMutation12$.loading,
      error = _useMutation12$.error;

  var updateCustomer = (0, _react.useCallback)(
  /*#__PURE__*/
  function () {
    var _ref2 = (0, _asyncToGenerator2["default"])(
    /*#__PURE__*/
    _regenerator["default"].mark(function _callee(_ref3) {
      var customerUpdates, idToken;
      return _regenerator["default"].wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              customerUpdates = _ref3.customerUpdates, idToken = _ref3.idToken;
              setLoading(true);
              _context.next = 4;
              return customerUpdate({
                variables: {
                  shopName: shopifyConfig.shopName,
                  idToken: idToken,
                  customerUpdates: customerUpdates
                }
              });

            case 4:
              setLoading(false);

            case 5:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function (_x) {
      return _ref2.apply(this, arguments);
    };
  }(), [shopifyConfig]);
  var updateCustomerPassword = (0, _react.useCallback)(
  /*#__PURE__*/
  function () {
    var _ref4 = (0, _asyncToGenerator2["default"])(
    /*#__PURE__*/
    _regenerator["default"].mark(function _callee2(customerId, resetToken, password) {
      var _ref5, data;

      return _regenerator["default"].wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              setLoading(true);
              _context2.next = 3;
              return customerPasswordUpdate({
                variables: {
                  shopName: shopifyConfig.shopName,
                  customerId: customerId,
                  resetToken: resetToken,
                  password: password
                }
              });

            case 3:
              _ref5 = _context2.sent;
              data = _ref5.data;
              setLoading(false);
              return _context2.abrupt("return", data.customerPasswordUpdate);

            case 7:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    return function (_x2, _x3, _x4) {
      return _ref4.apply(this, arguments);
    };
  }(), [shopifyConfig]);
  var activateCustomerAccount = (0, _react.useCallback)(
  /*#__PURE__*/
  function () {
    var _ref6 = (0, _asyncToGenerator2["default"])(
    /*#__PURE__*/
    _regenerator["default"].mark(function _callee3(customerId, activationToken, password) {
      var _ref7, data;

      return _regenerator["default"].wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              setLoading(true);
              _context3.next = 3;
              return customerAccountActivate({
                variables: {
                  shopName: shopifyConfig.shopName,
                  customerId: customerId,
                  activationToken: activationToken,
                  password: password
                }
              });

            case 3:
              _ref7 = _context3.sent;
              data = _ref7.data;
              setLoading(false);
              return _context3.abrupt("return", data.customerAccountActivate);

            case 7:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3);
    }));

    return function (_x5, _x6, _x7) {
      return _ref6.apply(this, arguments);
    };
  }(), [shopifyConfig]);
  var createSubscriber = (0, _react.useCallback)(
  /*#__PURE__*/
  function () {
    var _ref8 = (0, _asyncToGenerator2["default"])(
    /*#__PURE__*/
    _regenerator["default"].mark(function _callee4(_ref9) {
      var email;
      return _regenerator["default"].wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              email = _ref9.email;
              setLoading(true);
              _context4.next = 4;
              return subscriberCreate({
                variables: {
                  shopName: shopifyConfig.shopName,
                  email: email
                }
              });

            case 4:
              setLoading(false);

            case 5:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4);
    }));

    return function (_x8) {
      return _ref8.apply(this, arguments);
    };
  }(), [shopifyConfig]);
  var updateSubscriber = (0, _react.useCallback)(
  /*#__PURE__*/
  function () {
    var _ref10 = (0, _asyncToGenerator2["default"])(
    /*#__PURE__*/
    _regenerator["default"].mark(function _callee5(_ref11) {
      var customerUpdates, email;
      return _regenerator["default"].wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              customerUpdates = _ref11.customerUpdates, email = _ref11.email;
              setLoading(true);
              _context5.next = 4;
              return subscriberUpdate({
                variables: {
                  shopName: shopifyConfig.shopName,
                  email: email,
                  customerUpdates: customerUpdates
                }
              });

            case 4:
              setLoading(false);

            case 5:
            case "end":
              return _context5.stop();
          }
        }
      }, _callee5);
    }));

    return function (_x9) {
      return _ref10.apply(this, arguments);
    };
  }(), [shopifyConfig]);
  var retrieveCustomerAccessToken = (0, _react.useCallback)(function (_ref12) {
    var idToken = _ref12.idToken;

    var storedToken = _store["default"].get("next:accessToken");

    var storedTokenExpiresAt = _store["default"].get("next:accessToken:expiresAt"); // TODO: verify expiration


    return storedToken ? storedToken : createCustomerAccessToken({
      idToken: idToken
    });
  }, [shopifyConfig]);
  var createCustomerAccessToken = (0, _react.useCallback)(
  /*#__PURE__*/
  function () {
    var _ref13 = (0, _asyncToGenerator2["default"])(
    /*#__PURE__*/
    _regenerator["default"].mark(function _callee6(_ref14) {
      var idToken, _ref15, _ref15$data$customerA, accessToken, expiresAt;

      return _regenerator["default"].wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              idToken = _ref14.idToken;
              setLoading(true);
              _context6.next = 4;
              return customerAccessTokenCreate({
                variables: {
                  shopName: shopifyConfig.shopName,
                  idToken: idToken
                }
              });

            case 4:
              _ref15 = _context6.sent;
              _ref15$data$customerA = _ref15.data.customerAccessTokenCreate;
              accessToken = _ref15$data$customerA.accessToken;
              expiresAt = _ref15$data$customerA.expiresAt;

              _store["default"].set("next:accessToken", accessToken);

              _store["default"].set("next:accessToken:expiresAt", expiresAt);

              setLoading(false);

            case 11:
            case "end":
              return _context6.stop();
          }
        }
      }, _callee6);
    }));

    return function (_x10) {
      return _ref13.apply(this, arguments);
    };
  }(), [shopifyConfig]);
  var loading = isLoadingCustomer || isLoadingCustomerPassword || isLoadingCustomerActivate || isLoadingSubscriberCreate || isLoadingSubscriberUpdate || isLoadingCustomerAccessToken;
  (0, _react.useEffect)(function () {
    if (!isLoading) {
      var _authContext$authUser, _authContext$authUser2;

      /** if we've got a Firebase User and shopifyConfig but no customerAccessToken: fetch a customerAccessToken */
      if ((authContext === null || authContext === void 0 ? void 0 : authContext.authUser) && !(authContext === null || authContext === void 0 ? void 0 : (_authContext$authUser = authContext.authUser) === null || _authContext$authUser === void 0 ? void 0 : (_authContext$authUser2 = _authContext$authUser.roles) === null || _authContext$authUser2 === void 0 ? void 0 : _authContext$authUser2.GUEST) && !customerAccessToken && shopifyConfig) {
        var _authContext$authUser3;

        createCustomerAccessToken({
          idToken: authContext === null || authContext === void 0 ? void 0 : (_authContext$authUser3 = authContext.authUser) === null || _authContext$authUser3 === void 0 ? void 0 : _authContext$authUser3.idToken
        });
      }
    }
  }, [authContext, customerAccessToken, shopifyConfig]);
  var state = (0, _react.useMemo)(function () {
    return {
      isLoading: loading || isLoading,
      updateCustomer: updateCustomer,
      updateCustomerPassword: updateCustomerPassword,
      activateCustomerAccount: activateCustomerAccount,
      createSubscriber: createSubscriber,
      updateSubscriber: updateSubscriber,
      createCustomerAccessToken: createCustomerAccessToken,
      retrieveCustomerAccessToken: retrieveCustomerAccessToken,
      customerAccessToken: customerAccessToken,
      shopifyConfig: shopifyConfig,
      shopifyConfigLoading: !Boolean(shopifyConfig)
    };
  }, [loading, isLoading, updateCustomer, updateCustomerPassword, activateCustomerAccount, createSubscriber, updateSubscriber, createCustomerAccessToken, retrieveCustomerAccessToken, customerAccessToken, shopifyConfig]);
  var memoized = (0, _react.useMemo)(function () {
    return Boolean(shopifyConfig) ? children(shopifyConfig) : children(shopifyConfigs[0]);
  }, [shopifyConfig]);
  return _react["default"].createElement(FunctionsContext.Provider, {
    value: state
  }, memoized);
};

var _default = (0, _react.memo)(FunctionsProvider);

exports["default"] = _default;